<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2015 IMIA net based solutions (info@imia.de)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace IMIA\ImiaPageteaser\ViewHelpers\Backend;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Imaging\Icon;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Imaging\IconFactory;

/**
 * @package     imia_pageteaser
 * @subpackage  ViewHelpers
 * @author      David Frerich <d.frerich@imia.de>
 */
class IconViewHelper extends \TYPO3\CMS\Fluid\Core\ViewHelper\AbstractViewHelper
{
    /**
     * @param string $table
     * @param string $row
     * @param boolean $clickmenu
     * @return string
     */
    public function render($table = null, $row = null, $clickmenu = true)
    {
        $record = $this->renderChildren();
        if (!$table && is_object($record) && method_exists($record, 'getTable')) {
            $table = $record->getTable();
        }
        if (!$row && is_object($record) && method_exists($record, 'getRow')) {
            $row = $record->getRow();
        }

        $icon = '';
        if ($row && is_array($row) && $table) {
            $icon = $this->getIconFactory()->getIconForRecord('pages', [
                'uid'      => $row['uid'],
                'doktype'  => $row['doktype'],
                'hidden'   => $row['hidden'],
                'nav_hide' => $row['nav_hide'],
                'module'   => $row['module'],
            ], Icon::SIZE_SMALL);

            if ($clickmenu && $row['uid']) {
                $icon = BackendUtility::wrapClickMenuOnIcon($icon, $table, $row['uid'], 1, '', '');
            }
        }

        return $icon;
    }

    /**
     * @return IconFactory
     */
    protected function getIconFactory()
    {
        return GeneralUtility::makeInstance(IconFactory::class);
    }
}